namespace Tarpit;

public class WordPredictor
{
    private Dictionary<string, List<string>> wordFrequencies;
    
    public WordPredictor(string filePath)
    {
        LoadData(filePath);
    }
    
    private void LoadData(string filePath)
    {
        wordFrequencies = new Dictionary<string, List<string>>();

        try
        {
            using (StreamReader reader = new StreamReader(filePath))
            {
                string line;
                string previousWord = null;

                while ((line = reader.ReadLine()) != null)
                {
                    string[] words = line.ToLower().Split(new char[] { ' ', '.', ',', '!', '?', ';', ':', '\n', '\r', '\t' }, StringSplitOptions.RemoveEmptyEntries);

                    foreach (string word in words)
                    {
                        if (previousWord != null)
                        {
                            if (!wordFrequencies.ContainsKey(previousWord))
                            {
                                wordFrequencies[previousWord] = new List<string>();
                            }
                            wordFrequencies[previousWord].Add(word);
                        }
                        previousWord = word;
                    }
                }
            }
        }
        catch (FileNotFoundException)
        {
            Console.WriteLine($"Error: File not found at {filePath}");
            Environment.Exit(1); // Exit the program if the file isn't found.  Important!
        }
        catch (Exception ex)
        {
            Console.WriteLine($"An error occurred while reading the file: {ex.Message}");
            Environment.Exit(1);
        }
    }

    public string PredictNextWord(string inputWord)
    {
        if (!wordFrequencies.ContainsKey(inputWord.ToLower()))
        {
            return "Word not found in training data.";
        }

        List<string> nextWords = wordFrequencies[inputWord.ToLower()];

        // Find the most frequent next word
        string mostFrequentWord = nextWords
            .GroupBy(w => w)
            .OrderByDescending(g => g.Count())
            .First()
            .Key;

        return mostFrequentWord;
    }


    public static void Main(string[] args)
    {
        // Replace "your_text_file.txt" with the path to your text file.
        string filePath = "your_text_file.txt";

        WordPredictor predictor = new WordPredictor(filePath);

        Console.WriteLine("Enter a word:");
        string inputWord = Console.ReadLine();

        string predictedWord = predictor.PredictNextWord(inputWord);

        Console.WriteLine($"The most common word after '{inputWord}' is: '{predictedWord}'");

        Console.ReadKey(); // Keep the console window open until a key is pressed
    }
}